@@ -22,13 +22,19 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from __future__ import annotations
+from typing import List, Optional
 from enum import IntEnum
+import typing
 
 from gettext import gettext as _, ngettext
 import gi
 gi.require_version('Gst', '1.0')
 gi.require_version('GstPbutils', '1.0')
-from gi.repository import GLib, Gtk, Gio, GObject, Gst, GstPbutils
+from gi.repository import Gtk, Gio, GObject, Gst, GstPbutils
+
+if typing.TYPE_CHECKING:
+    from gnomemusic.application import Application
 
 
 class Playback(IntEnum):
@@ -53,7 +59,7 @@ class GstPlayer(GObject.GObject):
         "stream-start": (GObject.SignalFlags.RUN_FIRST, None, ())
     }
 
-    def __init__(self, application):
+    def __init__(self, application: Application) -> None:
         """Initialize the GStreamer player
 
         :param Application application: Application object
@@ -64,14 +70,15 @@ class GstPlayer(GObject.GObject):
 
         self._application = application
         self._duration = -1.
+        self._known_duration = False
         self._log = application.props.log
         self._seek = False
         self._tick = 0
 
         self._clock_id = 0
-        self._clock = None
+        self._clock: Optional[Gst.Clock] = None
 
-        self._missing_plugin_messages = []
+        self._missing_plugin_messages: List[Gst.Message] = []
         self._settings = application.props.settings
 
         self._player = Gst.ElementFactory.make('playbin3', 'player')
@@ -80,11 +87,12 @@ class GstPlayer(GObject.GObject):
         self._setup_replaygain()
 
         self._settings.connect(
-            'changed::replaygain', self._on_replaygain_setting_changed)
-        self._on_replaygain_setting_changed(
-            None, self._settings.get_value('replaygain'))
+            "changed::replaygain", self._on_replaygain_setting_changed)
+        self._settings.emit("changed", "replaygain")
 
         self._bus.connect('message::async-done', self._on_async_done)
+        self._bus.connect(
+            "message::duration-changed", self._on_duration_changed)
         self._bus.connect('message::error', self._on_bus_error)
         self._bus.connect('message::element', self._on_bus_element)
         self._bus.connect('message::eos', self._on_bus_eos)
@@ -120,7 +128,9 @@ class GstPlayer(GObject.GObject):
             self._log.message("Replay Gain is not available")
             return
 
-    def _on_replaygain_setting_changed(self, settings, value):
+    def _on_replaygain_setting_changed(
+            self, settings: Gio.Settings, key: str) -> None:
+        value = settings.get_value(key)
         if value:
             self._player.set_property("audio-filter", self._filter_bin)
         else:
@@ -134,12 +144,19 @@ class GstPlayer(GObject.GObject):
             self._seek = False
             self.emit("seek-finished")
 
-    def _query_duration(self):
-        success, duration = self._player.query_duration(
+        if not self._known_duration:
+            self._query_duration()
+
+    def _on_duration_changed(self, bus: Gst.Bus, message: Gst.Message) -> None:
+        self._query_duration()
+
+    def _query_duration(self) -> None:
+        self._known_duration, duration = self._player.query_duration(
             Gst.Format.TIME)
 
-        if success:
+        if self._known_duration:
             self.props.duration = duration / Gst.SECOND
+            self._log.debug("duration changed: {}".format(self.props.duration))
         else:
             self.props.duration = duration
 
@@ -151,7 +168,7 @@ class GstPlayer(GObject.GObject):
             self._clock.get_time(), 1 * Gst.SECOND)
         self._clock.id_wait_async(self._clock_id, self._on_clock_tick, None)
 
-    def _destroy_clock_tick(self):
+    def _destroy_clock_tick(self) -> None:
         if (self._clock_id > 0
                 and self._clock is not None):
             self._clock.id_unschedule(self._clock_id)
@@ -171,14 +188,9 @@ class GstPlayer(GObject.GObject):
             self._missing_plugin_messages.append(message)
 
     def _on_bus_stream_start(self, bus, message):
-        def delayed_query():
-            self._query_duration()
-            self._tick = 0
-            self.emit("stream-start")
-
-        # Delay the signalling slightly or the new duration will not
-        # have been set yet.
-        GLib.timeout_add(1, delayed_query)
+        self._query_duration()
+        self._tick = 0
+        self.emit("stream-start")
 
     def _on_state_changed(self, bus, message):
         if message.src != self._player:
@@ -283,7 +295,9 @@ class GstPlayer(GObject.GObject):
 
         return position
 
-    @GObject.Property(type=float)
+    @GObject.Property(
+        type=float, flags=GObject.ParamFlags.READWRITE
+        | GObject.ParamFlags.EXPLICIT_NOTIFY)
     def duration(self):
         """Total duration of current media
 
@@ -304,7 +318,9 @@ class GstPlayer(GObject.GObject):
 
         For internal use only.
         """
-        self._duration = duration
+        if duration != self._duration:
+            self._duration = duration
+            self.notify("duration")
 
     def seek(self, seconds):
         """Seek to position
